import base64

try:
    import json
except ImportError:
    try:
        import simplejson as json
    except ImportError:
        import django.utils.simplejson as json

import boto.sqs.message

from django.contrib.contenttypes.models import ContentType


class ModelInstanceMessage(boto.sqs.message.RawMessage):
    """SQS Message class that returns 
    """
    def __init__(self, queue=None, instance=None):
        boto.sqs.message.RawMessage.__init__(
            self, queue=queue, body=instance)

    def encode(self, value):
        ct = ContentType.objects.get_for_model(value)
        return base64.b64encode(
            json.dumps(
                (ct.app_label, ct.model, value.pk)))

    def decode(self, value):
        try:
            app_label, model, pk = json.loads(base64.b64decode(value))
        except Exception, e:
            self.__reason = "Error decoding payload: %s" % e
            return None
            
        try:
            ct = ContentType.objects.get(app_label=app_label, model=model)
        except ContentType.DoesNotExist:
            self.__reason = "Invalid content type."
            return None

        cls = ct.model_class()
        try:
            return cls.objects.get(pk=pk)
        except cls.DoesNotExist:
            self.__reason = "%s.%s %r does not exist" % (
                cls.__module__, cls.__name__, pk)
            return None

    def get_body(self):
        rv = boto.sqs.message.RawMessage.get_body(self)
        if rv is not None:
            return rv
        raise ValueError(self.__reason)

    def get_instance(self):
        return self.get_body()



import boto.sqs.connection

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from registered_queue import RegisteredQueue, TimedOut, RestartLater


# ensure settings are there

try:
    from settings import AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY
    if settings.DEBUG:
        from settings import SQS_QUEUE_PREFIX
except Exception, e:
    raise ImproperlyConfigured("Misconfigured: %s" % e)

# registry

queues = {}

# convenience

def register(queue_name, fn=None, **kwargs):
    rv = RegisteredQueue(queue_name, fn, **kwargs)
    queues[queue_name] = rv
    return rv


def receiver(queue_name=None, **kwargs):
    """Registers decorated function as SQS message receiver."""
    def _decorator(fn):
        qn = queue_name or '%s__%s' % (
            fn.__module__.replace('.','__'), fn.__name__ )
        return register(qn, fn, **kwargs).get_receiver_proxy()
    return _decorator


def send(queue_name, message=None, suffix=None, **kwargs):
    queues[queue_name].send(message, suffix, **kwargs)



from django.core.management.base import NoArgsCommand

import django_sqs


class Command(NoArgsCommand):
    help = "Provides information about used SQS queues and the number of items in each of them."

    def handle_noargs(self, **options):
        print
        print "Active SQS queues"
        print "-----------------"
        print
        for label, queue in django_sqs.queues.items():
            q = queue.get_queue()
            print "%-30s: %4s" % (label, q.count())
            if queue.suffixes:
                for suffix in queue.suffixes:
                    q = queue.get_queue(suffix)
                    print " .%-28s: %4s" % (suffix, q.count())



import time

from django.http import HttpResponse
from django.views.decorators.cache import cache_page

import django_sqs

@cache_page(5*60)
def status(request):
    """Returns a simple plain text rendering of queue status."""
    response_text = ['Queue status as of %s GMT\n' % time.strftime("%F %T", time.gmtime())]
    for label, queue in django_sqs.queues.items():
        q = queue.get_queue()
        response_text.append( "%s\t%s" % (label, q.count()) )
    
    return HttpResponse('\n'.join(response_text), mimetype='text/plain')



import logging
import signal
import time
from warnings import warn

import boto.sqs.message

from django.conf import settings

try:
    from django.db import connections
    CONNECTIONS = connections.all()
except ImportError:
    from django.db import connection
    CONNECTIONS = (connection, )

class _NullHandler(logging.Handler):
    def emit(self, record):
        pass

if settings.DEBUG:
    boto_debug = 1
else:
    boto_debug=0

DEFAULT_VISIBILITY_TIMEOUT = getattr(
    settings, 'SQS_DEFAULT_VISIBILITY_TIMEOUT', 60)

POLL_PERIOD = getattr(
    settings, 'SQS_POLL_PERIOD', 10)


class TimedOut(Exception):
    """Raised by timeout handler."""
    pass

def sigalrm_handler(signum, frame):
    raise TimedOut()


class RestartLater(Exception):
    """Raised by receivers to stop processing and leave message in queue."""
    pass


class UnknownSuffixWarning(RuntimeWarning):
    """Unknown suffix passed to a registered queue"""
    pass


class RegisteredQueue(object):

    class ReceiverProxy(object):
        """Callable object that sends message via appropriate SQS queue.

        Available attributes:
        - direct - inner (decorated) function
        - registered_queue - a RegisteredQueue instance for this queue
        """
        def __init__(self, registered_queue):
            self.registered_queue = registered_queue
            self.direct = registered_queue.receiver

        def __call__(self, message=None, **kwargs):
            self.registered_queue.send(message, **kwargs)

    def __init__(self, name,
                 receiver=None, visibility_timeout=None, message_class=None,
                 timeout=None, delete_on_start=False, close_database=False,
                 suffixes=()):
        self._connection = None
        self.name = name
        self.receiver = receiver
        self.visibility_timeout = visibility_timeout or DEFAULT_VISIBILITY_TIMEOUT
        self.message_class = message_class or boto.sqs.message.Message
        self.queues = {}
        self.timeout = timeout
        self.delete_on_start = delete_on_start
        self.close_database = close_database
        self.suffixes = suffixes

        if self.timeout and not self.receiver:
            raise ValueError("timeout is meaningful only with receiver")

        if not issubclass(self.message_class, boto.sqs.message.RawMessage):
            raise ValueError(
                "%s is not a subclass of boto.sqs.message.RawMessage"
                % self.message_class)

        self.prefix = getattr(settings, 'SQS_QUEUE_PREFIX', None)

        self._log = logging.getLogger('django_sqs.queue.%s' % self.name)
        self._log.addHandler(_NullHandler())
        self._log.info("Using queue %s" % self.full_name())

    def full_name(self, suffix=None):
        name = self.name
        if suffix:
            if suffix not in self.suffixes:
                warn("Unknown suffix %s" % suffix, UnknownSuffixWarning)
            name = '%s__%s' % ( name, suffix )
        if self.prefix:
            return '%s__%s' % (self.prefix, name)
        else:
            return name

    def get_connection(self):
        if self._connection is None:
            self._connection = boto.sqs.connection.SQSConnection(
                settings.AWS_ACCESS_KEY_ID,
                settings.AWS_SECRET_ACCESS_KEY,
                debug=boto_debug)
        return self._connection

    def get_queue(self, suffix=None):
        if suffix not in self.queues:
            self.queues[suffix] = self.get_connection().create_queue(
                self.full_name(suffix), self.visibility_timeout)
            self.queues[suffix].set_message_class(self.message_class)
        return self.queues[suffix]

    def get_receiver_proxy(self):
        return self.ReceiverProxy(self)

    def send(self, message=None, suffix=None, **kwargs):
        q = self.get_queue(suffix)
        if message is None:
            message = self.message_class(**kwargs)
        else:
            if not isinstance(message, self.message_class):
                raise ValueError('%r is not an instance of %r' % (
                    message, self.message_class))
        q.write(message)

    def receive(self, message):
        if self.receiver is None:
            raise Exception("Not configured to received messages.")
        if self.timeout:
           signal.alarm(self.timeout)
           signal.signal(signal.SIGALRM, sigalrm_handler)
        try:
            self.receiver(message)
        finally:
            if self.timeout:
                try:
                    signal.alarm(0)
                    signal.signal(signal.SIGALRM, signal.SIG_DFL)
                except TimedOut:
                    # possible race condition if we don't cancel the
                    # alarm in time.  Now there is no race condition
                    # threat, since alarm already rang.
                    signal.alarm(0)
                    signal.signal(signal.SIGALRM, signal.SIG_DFL)
            if self.close_database:
                for connection in CONNECTIONS:
                    print 'Closing', connection
                    connection.close()


    def receive_single(self, suffix=None):
        """Receive single message from the queue.

        This method is here for debugging purposes.  It receives
        single message from the queue, processes it, deletes it from
        queue and returns (message, handler_result_value) pair.
        """
        q = self.get_queue(suffix)
        mm = q.get_messages(1)
        if mm:
            if self.delete_on_start:
                q.delete_message(mm[0])
            rv1 = self.receive(mm[0])
            if not self.delete_on_start:
                q.delete_message(mm[0])
            return (mm[0], rv1)

    def receive_loop(self, message_limit=None, suffix=None):
        """Run receiver loop.

        If `message_limit' number is given, return after processing
        this number of messages.
        """
        q = self.get_queue(suffix)
        i = 0
        while True:
            if message_limit:
                i += 1
                if i > message_limit:
                    return
            mm = q.get_messages(1)
            if not mm:
                time.sleep(POLL_PERIOD)
            else:
                for m in mm:
                    try:
                        if self.delete_on_start:
                            q.delete_message(m)
                        self.receive(m)
                    except KeyboardInterrupt, e:
                        raise e
                    except RestartLater:
                        self._log.debug("Restarting message handling")
                    except:
                        try:
                            body = repr(m.get_body())
                        except Exception, e:
                            body = "(cannot run %r.get_body(): %s)" % (m, e)
                        self._log.exception(
                            "Caught exception in receive loop for %s %s" % (
                                m.__class__, body))
                        if not self.delete_on_start:
                            q.delete_message(m)
                    else:
                        if not self.delete_on_start:
                            q.delete_message(m)



#!/usr/bin/env python

try:
	from setuptools import setup, find_packages
except ImportError:
	from ez_setup import use_setuptools
	use_setuptools()
	from setuptools import setup, find_packages

setup(
    name='django-sqs',
    version="0.1",
    description='django FormFields using the Chosen javascript plugin for jQuery',
    author='Maciej Pasternacki',
    author_email='maciej@pasternacki.net',
    url='https://github.com/Fandekasp/django-sqs',
    packages=find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
	include_package_data=True,
	zip_safe=False,
)




from optparse import make_option
import time

from django.conf import settings
from django.core.management.base import BaseCommand

import django_sqs

WAIT_TIME = getattr(settings, 'SQS_WAIT_TIME', 60)
WAIT_CYCLES = getattr(settings, 'SQS_WAIT_CYCLES', 1)

class Command(BaseCommand):
    help = "Wait until queues registered with django_sqs are empty" \
           " and all messages are processed."
    args = '[queue_name [queue_name [...]]]'

    option_list = BaseCommand.option_list + (
        make_option('--wait-time', '-t',
                    type='int', dest="wait_time", metavar='SECONDS',
                    default=WAIT_TIME,
                    help = "Time to sleep between queue count checks."
                    " Default is %d." % WAIT_TIME),
        make_option('--wait-cycles', '-c',
                    type='int', dest="wait_cycles", metavar='N',
                    default=WAIT_CYCLES,
                    help="Number of wait-time periods for which queues"
                    " must be empty to assume all messages have been"
                    " processed and stop waiting.  Default is %d, set"
                    " to 0 to stop waiting immediately when queue is empty"
                    % WAIT_CYCLES),
        )

    def handle(self, *queue_names, **options):
        self.validate()
        if not queue_names:
            queue_names = django_sqs.queues.keys()
        verbosity = int(options.get('verbosity', 1))

        empty_cycles = 0
        while True:
            empty = True
            for qname in queue_names:
                q = django_sqs.queues[qname].get_queue()
                c = q.count()
                if c > 0:
                    empty = False
                    if verbosity > 1:
                        print "Queue %s has %d messages, keep waiting." % (
                            qname, c)
                    break

            if empty:
                empty_cycles += 1
                if empty_cycles > options['wait_cycles']:
                    if verbosity > 1:
                        print '%d empty cycles, finishing.' % (empty_cycles - 1)
                    break
                else:
                    if verbosity > 1:
                        print '%d empty cycles, less than %d, still waiting.' % (
                            empty_cycles - 1, options['wait_cycles'])
            else:
                empty_cycles = 0

            time.sleep(options['wait_time'])



import logging
import os
from optparse import make_option
import signal

from django.core.management.base import BaseCommand

import django_sqs

# null handler to avoid warnings
class _NullHandler(logging.Handler):
    def emit(self, record):
        pass

# signal name dict for logging
_signals = {}
for name in dir(signal):
    if name.startswith('SIG'):
        _signals[getattr(signal, name)] = name
def _status_string(status):
    "Pretty status description for exited child."

    if os.WIFSIGNALED(status):
        return "Terminated by %s (%d)" % (
            _signals.get(os.WTERMSIG(status), "unknown signal"),
            os.WTERMSIG(status))

    if os.WIFEXITED(status):
        return "Exited with status %d" % os.WEXITSTATUS(status)

    if os.WIFSTOPPED(status):
        return "Stopped by %s (%d)" % (
            _signals.get(os.WSTOPSIG(status), "unknown signal"),
            os.WSTOPSIG(status))

    if os.WIFCONTINUED(status):
        return "Continued from stop"

    return "Unknown reason (%r)" % status

class Command(BaseCommand):
    help = "Run Amazon SQS receiver for queues registered with django_sqs."
    args = '[queue_name [queue_name [...]]]'
    option_list = BaseCommand.option_list + (
        make_option('--daemonize',
                    action='store_true', dest='daemonize', default=False,
                    help='Fork into background as a daemon.'),
        make_option('--daemon-stdout-log',
                    dest='stdout_log', default='/dev/null',
                    help="Log daemon's standard output stream to a file"),
        make_option('--daemon-stderr-log',
                    dest='stderr_log', default='/dev/null',
                    help="Log daemon's standard error stream to a file"),
        make_option('--suffix', dest='suffix', default=None, metavar='SUFFIX',
                    help="Append SUFFIX to queue name."),
        make_option('--pid-file',
                    dest='pid_file', default='',
                    help="Store process ID in a file"),
        make_option('--message-limit',
                    dest='message_limit', default=None, type='int',
                    metavar='N', help='Exit after processing N messages'),
        )

    def handle(self, *queue_names, **options):
        self.validate()

        if not queue_names:
            queue_names = django_sqs.queues.keys()

        if options.get('daemonize', False):
            from django.utils.daemonize import become_daemon
            become_daemon(out_log=options.get('stdout_log', '/dev/null'),
                          err_log=options.get('stderr_log', '/dev/null'))

        if options.get('pid_file', ''):
            with open(options['pid_file'], 'w') as f:
                f.write('%d\n' % os.getpid())

        if len(queue_names) == 1:
            self.receive(queue_names[0],
                         suffix=options.get('suffix'),
                         message_limit=options.get('message_limit', None))
        else:
            # fork a group of processes.  Quick hack, to be replaced
            # ASAP with something decent.
            _log = logging.getLogger('django_sqs.runreceiver.master')
            _log.addHandler(_NullHandler())

            # Close the DB connection now and let Django reopen it when it
            # is needed again.  The goal is to make sure that every
            # process gets its own connection
            from django.db import connection
            connection.close()

            os.setpgrp()
            children = {}               # queue name -> pid
            for queue_name in queue_names:
                pid = self.fork_child(queue_name,
                                      options.get('message_limit', None))
                children[pid] = queue_name
                _log.info("Forked %s for %s" % (pid, queue_name))

            while children:
                pid, status = os.wait()
                queue_name = children[pid]
                _log.error("Child %d (%s) exited: %s" % (
                    pid, children[pid], _status_string(status) ))
                del children[pid]

                pid = self.fork_child(queue_name)
                children[pid] = queue_name
                _log.info("Respawned %s for %s" % (pid, queue_name))

    def fork_child(self, queue_name, message_limit=None):
        pid = os.fork()
        if pid:                         # parent
            return pid
        # child
        _log = logging.getLogger('django_sqs.runreceiver.%s' % queue_name)
        _log.addHandler(_NullHandler())
        _log.info("Start receiving.")
        self.receive(queue_name, message_limit=message_limit)
        _log.error("CAN'T HAPPEN: exiting.")
        raise SystemExit(0)

    def receive(self, queue_name, message_limit=None, suffix=None):
        rq = django_sqs.queues[queue_name]
        if rq.receiver:
            if message_limit is None:
                message_limit_info = ''
            else:
                message_limit_info = ' %d messages' % message_limit

            print 'Receiving%s from queue %s%s...' % (
                message_limit_info, queue_name,
                ('.%s' % suffix if suffix else ''),
                )
            rq.receive_loop(message_limit=message_limit,
                            suffix=suffix)
        else:
            print 'Queue %s has no receiver, aborting.' % queue_name



import sys

from django.core.management.base import BaseCommand

import django_sqs


class Command(BaseCommand):
    help = "Clean Amazon SQS queues registered with django_sqs."
    args = '[queue_name [queue_name [...]]]'

    def handle(self, *queue_names, **options):
        self.validate()

        if not queue_names:
            queue_names = django_sqs.queues.keys()

        for queue_name in queue_names:
            q = django_sqs.queues[queue_name].get_queue()
            print 'Clearing queue %s...' % queue_name,
            sys.stdout.flush()
            n = q.clear()
            print '%d deleted.' % n

        
